require "/var/lib/asterisk/agi-bin/samyxml.pl";

sub get_permissions
{
	# take in permissions xml
	return SamyXMLin("/etc/asterisk/permissions.xml");
}

sub get_ast_hdrs
{
	my %info;

	# get asterisk headers
	while (<STDIN>)
	{
		$info{$1} = $2 if /^(.*): (.*)$/;
		dbg("$1 = $2");

		last if /^\r?\n/;
	}
	$info{agi_channel} =~ s/-.*$//;

	return %info;
}


sub ssend
{
	my $data = shift;
	print STDOUT "$data\n";
	my $line = <STDIN>;

	return $line;
}

# debug command
sub dbg
{
	my $log = shift;
	chomp($log);
	open F, ">>/tmp/agi.log";
	print F localtime() . ": $log\n";
	close(F);
}

# returns the CallerID (extension number) of the person paging
sub get_cid
{
	my $info = shift;

	my $cid = $info->{agi_callerid};
	$cid =~ s/^.*<(\d+)>$/$1/;

	if (!$cid)
	{
		die "no valid callerid!\n";
	}

	return $cid;
}

# verify user has proper rights to perform function
sub check_rights
{
	my ($ref, $ext, $exts, $right_name, $must_be_group, $must_be_individual) = @_;

	# find out who's allowed
	my %allowed = map { $_ => 1 } get_exts_from_group($ref, "ext->e${ext}->$right_name");

	# make sure we have permission
	foreach (keys %{$exts})
	{
		
		unless (
			($must_be_group      && $exts->{$_} == 1 && $allowed{$_}) ||
			($must_be_individual && $exts->{$_} == 0 && $allowed{$_}) ||
			(!$must_be_group && !$must_be_individual && $allowed{$_})
		)
		{
			# not allowed
			return 0;
		}
	}

	return 1;
}

# get extensions within allowed groups
sub get_exts_from_group
{
	my ($ref, @etc) = @_;

	my @groups = get_ref($ref, @etc);

	# get the extensions for each group
	return map { get_ref($ref, "ext_groups->g$_") } @groups;
}

# sweet function to return a list of shiz from refs
sub get_ref
{
	my ($ref, @list) = @_;

	@list = map { split("->") } @list;

	my $var = $ref;
	foreach (@list)
	{
		if (ref($var) eq "HASH")
		{
			$var = $var->{$_};
		}
		else
		{
			$var = undef;
			last;
		}
	}

	my @list = ref($var) eq "ARRAY" ? @{$var} : ($var);
	foreach (@list)
	{
		s/R\((\d+),(\d+)\)/join(",", $1 .. $2)/eg;
	}
	return map { split(",") } @list;
}

# produce a list of extensions from extensions/extension groups
sub get_exts
{
	my ($ref, @exts) = @_;

	my %group_exts;
	if (ref($ref->{groups}) eq "HASH")
	{
		foreach my $key (keys %{$ref->{groups}})
		{
			if ($key eq "name")
			{
				$group_exts{substr($ref->{groups}->{name}, 2)} = substr($ref->{groups}->{groupid}, 1);
				last;
			}
			elsif ($key =~ /^eg(\d+)$/)
			{
				$group_exts{$1} = substr($ref->{groups}->{$key}->{groupid}, 1);
			}
		}
	}

	my %struct;
	foreach my $ext (@exts)
	{
		# are we a group?
		if ($group_exts{$ext})
		{
			foreach (get_ref($ref, "ext_groups->g$group_exts{$ext}"))
			{
				$struct{$_} = 1;
			}
		}
		# not a group
		else
		{
			$struct{$ext} = 0;
		}
	}

	return \%struct;
}

1;
